#include <avr\io.h>
#include <avr\iom16.h>
#include <inttypes.h>
#include <avr/delay.h>  

#define EEPROMGROESSE 512					//Gre des EEPROM's ATMEGA16 = 512 Byte

unsigned int Adresse_alt = 0, adr = 0;

void uart_tx(unsigned char Byte)			//9600Baud
{	UCSRB |= (1<<TXEN);
	UCSRC |= (1<<URSEL) | (3<<UCSZ0);
	UBRRH = 0;
	UBRRL = 51;
	while(!(UCSRA & (1<<UDRE)));
	UDR = Byte;
}

void char_to_bin(char wert)
{	int i;
	char Buffer[8];
	for (i = 0; i <= 8; i++) 
		Buffer[i] = (wert & (1 << (7 - i))) ? '1' : '0'; 
	Buffer[8] = '\0'; 						//return Buffer[8];
}

uint16_t ADU_KANAL(uint8_t mux)
{	uint8_t i;										//Indexvariable
	uint16_t result =0;								//Ergebnis

	ADCSRA = (1<<ADEN) | (1<<ADPS2) | (1<<ADPS1); //Wandler einschalten, Wandlungsfrequenz = 125kHz
	ADMUX = mux;									//Gewnschter Kanal
	ADCSRA |= (1<<ADSC);							//Wandlung starten (single step)
	while (ADCSRA & (1<<ADSC));					//Warten bis Wandlung abgeschlossen
	
	for (i=0; i<4; i++)							//4 Wandlungen
	{	ADCSRA |= (1<<ADSC);						//Wandlung starten (single step)
		while (ADCSRA & (1<<ADSC));				//Warten bis Wandlung abgeschlossen
		result += ADCW;								//Ergebnisse aufsummieren
	}
	ADCSRA &= ~(1<<ADEN);							//Wandler wieder ausschalten
	result /=4;										//Mittelwert der 4 Wandlungen
	return result;									//Rckgabewert
}

void EEPROM_schreiben(char Byte)
{	adr = Adresse_alt;
	while (EECR & (1<<EEWE));						//Warten bis eventuell vorheriger Schreibvorgang abgeschlossen ist
	EEAR = (adr);									//Adresse in SFR schreiben
	EEDR = (Byte);									//Byte in SFR schreiben
	EECR |= (1<<EEMWE);							//Schreibschutz aufheben
	EECR |= (1<<EEWE);								//Byte schreiben
	adr++;
	Adresse_alt = adr;
}
void EEPROM_set_adr(unsigned int Adresse)
{	Adresse_alt = Adresse;
	
}
void EEPROM_schreiben_adr(unsigned int Adresse, char Byte)
{	Adresse = Adresse_alt;
	while (EECR & (1<<EEWE));						//Warten bis eventuell vorheriger Schreibvorgang abgeschlossen ist
	EEAR = (Adresse);								//Adresse in SFR schreiben
	EEDR = (Byte);									//Byte in SFR schreiben
	EECR |= (1<<EEMWE);							//Schreibschutz aufheben
	EECR |= (1<<EEWE);								//Byte schreiben
}

char EEPROM_lesen(unsigned int Adresse)		
{	char Byte; 										//Rckgabewert
	unsigned int temp;
	temp=EEAR;
	while(EECR & (1<<EERE));						//Werten bis vorheriger LEsevorgan abgeschlossen ist
	EEAR = (Adresse);								//Adresse in SFR schreiben
	EECR |= (1<<EERE);								//Byte von Adresse lesen (EEPROM Read Enable)
	Byte = EEDR;									//Byte aus SFR holen
	return Byte;									//als Rckgabewert der Funktion
	EEAR=temp;
}
void EEPROM_clr(void)
{	unsigned int i;
	for(i=0; i<=(EEPROMGROESSE-1); i++)
	{	while (EECR & (1<<EEWE));						//Warten bis eventuell vorheriger Schreibvorgang abgeschlossen ist
		EEAR = (i);								//Startadresse in SFR schreiben
		EEDR = 0xFF;									//Byte in SFR schreiben
		EECR |= (1<<EEMWE);							//Schreibschutz aufheben
		EECR |= (1<<EEWE);								//Byte schreiben
	}
	EEAR = 0x00;
	Adresse_alt=0x00;
}
void EEPROM_loeschen(unsigned int Startadresse)
{	unsigned int adr_del, bereich;
	bereich = (EEPROMGROESSE - Startadresse);
	
	for(adr_del=Startadresse; adr_del<=(EEPROMGROESSE-1); adr_del++) 
	{	while (EECR & (1<<EEWE));						//Warten bis eventuell vorheriger Schreibvorgang abgeschlossen ist
		EEAR = (adr_del);								//Startadresse in SFR schreiben
		EEDR = 0xFF;									//Byte in SFR schreiben
		EECR |= (1<<EEMWE);							//Schreibschutz aufheben
		EECR |= (1<<EEWE);								//Byte schreiben
	}
	Adresse_alt=Startadresse;
}

void EEPROM_string(char *s)
{	adr = Adresse_alt;
	do
	{	while(EECR & (1<<EEWE));
		EEAR = (adr);
		EEDR = *s;
		EECR |=(1<<EEMWE);
		EECR |=(1<<EEWE);
		s++;
		adr++;
	}while(*s);
	Adresse_alt = adr;
}

void EEPROM_string_adr(unsigned int Adresse, char *s)
{	if(Adresse<Adresse_alt)		adr=Adresse_alt;
	else	adr = Adresse;
	do
	{	while(EECR & (1<<EEWE));
		EEAR = (adr);
		EEDR = *s;
		EECR |=(1<<EEMWE);
		EECR |=(1<<EEWE);
		s++;
		adr++;
	}while(*s);
	Adresse_alt = adr;
}

long int speicherpruefung(long int EEP_SIZE)		   					//UP zur Prfung des belegten Speichers
{	long int a=0;														
	unsigned char Byte[EEP_SIZE-1];
	for(a=0; a<=(EEP_SIZE-1); a++)									//Schleife zum Auslesen des EEPROMs
	{	Byte[a]= EEPROM_lesen(a);										//EEPROM Byte in Array einlesen
		if (Byte[a] == 0xFF)											//Wenn ein Byte 0xFF (leer) ist dann
		{	Byte[a+1] = EEPROM_lesen(a+1);								//Darauffolgendes Byte einlesen +1
			if(Byte[a+1] == 0xFF)										//Ist dieses auch 0xFF (leer)
			{	Byte[a+2] = EEPROM_lesen(a+2);							//Nchstes Byte einlesen +2
				if(Byte[a+2] == 0xFF)									//usw. das Ganze 5 Mal.
				{	Byte[a+3] = EEPROM_lesen(a+3);
					if(Byte[a+3] == 0xFF)
					{	Byte[a+4] = EEPROM_lesen(a+4);
						if(Byte[a+4] ==0xFF)
						{	return a;									//Rckgabe, erste leere Adresse
							break;
						}
					}
				}
			}
		}
	}
}
