//Funktionen zum Ansteuern eines EEproms ber den IC-Bus
//EEPROM ST M24512-W	(64kByte zu 8Bit)
//November 2006

#include <avr/io.h>
#include "i2c_eep.h"

//*****************************************************************************************************************
//---------------------------- IC-Hilfsfunktionen ---------------------------------------------------------------------

int i2c_ini(void)								//100kHz
{	TWBR = FAKTOR;								//TWI-Bitrate einstellen					
	TWSR = TEILER;								//Vorteiler einstellen
	return 0;
}

int i2c_start(void)
{	TWCR = (1<<TWINT)|(1<<TWSTA)|(1<<TWEN);	//Startbedingung
	while (!(TWCR & (1<<TWINT)));				//Warten bis fertig
	return 0;
}

int i2c_stop(void)
{	TWCR = (1<<TWINT)|(1<<TWSTO)|(1<<TWEN);	//Stoppbedingung
	return 0;
}

int i2c_write_byte(unsigned char byte)
{	TWDR = byte;								//Daten laden
	TWCR = (1<<TWINT)|(1<<TWEN);				//Senden
	while (!(TWCR & (1<<TWINT)));				//Warten bis Bit Null ist, dann fertig
	return 0;
}

unsigned char i2c_read_byte(unsigned char ack_flag)
{	unsigned char daten;

	if(ack_flag)
	{	TWCR = (1<<TWINT)|(1<<TWEN);				//Empfang starten
		while (!(TWCR & (1<<TWINT)));				//Warten bis Bit Null ist, dann fertig
		daten = TWDR;								//daten aus Schiebregister lesen
		TWCR = (1<<TWINT)|(1<<TWEA)|(1<<TWEN);	//ACK_impuls generieren (ACK)
		while (!(TWCR & (1<<TWINT)));				//Warten bis Bit Null ist, dann fertig
	}
	else
	{	TWCR = (1<<TWINT)|(1<<TWEN);				//Empfang starten
		while (!(TWCR & (1<<TWINT)));				//Warten bis Bit Null ist, dann fertig
		daten = TWDR;								//daten aus Schiebregister lesen
		TWCR = (1<<TWINT)|(1<<TWEN);				//keinen ACK_impuls generieren (NACK)
		while (!(TWCR & (1<<TWINT)));				//Warten bis Bit Null ist, dann fertig
	}
	return daten;
}

//*****************************************************************************************************************
//---------------------------- IC-Haupfunktionen -----------------------------------------------------------------

//Funktion schreibt ein Byte an eine Gerteadresse
//bergabeparameter: Gerteadresse (ohne RW), Byte
//Rckgabeparameter: 0 wenn alles ok
int i2c_write_byte_adr(unsigned char ADR, unsigned char byte)
{	i2c_start();
	i2c_write_byte(ADR);						
	i2c_write_byte(byte);
	i2c_stop();
	return 0;
}
//Funktion liest ein Byte von einer Gerteadresse
//bergabeparameter: Gerteadresse(ohne RW), 1 wenn beim lesen auf ein ACK gewartet werden soll, 0 wenn NACK
//Rckgabeparameter: gelsenes Byte
unsigned char i2c_read_byte_adr(unsigned char ADR, unsigned char ack_flag)
{	unsigned char temp;
	i2c_start();
	i2c_write_byte(ADR|0x01);							//Bei Lesen RW=1
	temp = i2c_read_byte(ack_flag);
	i2c_stop();
	return temp;
}

//*****************************************************************************************************************
//---------------------------- EEPROM-Funktionen ------------------------------------------------------------------


//Funktion liest die Speicheradresse aus (nicht korrekt, keine Ahnung warum nur ein Byte zurckgegeben wird 
//Rckgabe: Adresse (uchar)
unsigned char i2c_eep_readadr(unsigned char DEV_ADR)
{	unsigned char data_out;
	i2c_start();												//UP Startbedingung
	i2c_write_byte(DEV_ADR|0x01);								//Byte lesen (Gerteadresse) R/W = 1
	data_out=i2c_read_byte(0);									//Byte lesen (NACK, 0)
	i2c_stop();
	return data_out;
}


//Funtkion schreibt ein Byte an eine bestimmte Stelle im EEprom
//Rckgabe: 0 bei korrekter Ausfhrung
int i2c_eep_write_byte(unsigned char DEV_ADR, unsigned int EEP_ADR, unsigned char byte)
{	i2c_start();
	i2c_write_byte(DEV_ADR);									//Sendet Gerteadresse RW=0, schreiben
	i2c_write_byte((unsigned char)(EEP_ADR>>8));				//Sendet MSB Teil der Speicheradresse
	i2c_write_byte((unsigned char)(EEP_ADR));					//Sendet LSB Teil der Spericheradresse
	i2c_write_byte(byte);
	i2c_stop();
	return 0;
}


//Funktion schreibt eine Page (max 128 Bytes) an eine bestimmte Stelle im EEprom
//bergabe: Gerteadresse(ohne RW), Adresse im EEPROM, Adresse (&) des Bytes, Anzahl der Bytes
//Rckgabe: 1 wenn Zu viele Bytes gesendet werden sollten
//			0 wenn alles OK

int i2c_eep_write_page(unsigned char DEV_ADR, unsigned int EEP_ADR, unsigned char *byte, unsigned char n)
{	unsigned char i;
	if(n>128)	return 1;
	else
	{	i2c_start();
		i2c_write_byte(DEV_ADR);									//Sendet Gerteadresse
		i2c_write_byte((unsigned char)(EEP_ADR>>8));				//Sendet MSB Teil der Speicheradresse
		i2c_write_byte((unsigned char)(EEP_ADR));					//Sendet LSB Teil der Spericheradresse
		for(i=0; i<=n; i++)
		{	i2c_write_byte(*byte++);
			//*byte++;
		}
		i2c_stop();
	}
	return 0;
}

//Funktion zum Lesen eines Speicherblocks
//bergabe: Gerteadresse (ohne RW), Speicheradresse, Anzahl Lesenden Bytes, 
//Rckgabe: Array in das geschrieben werden soll


unsigned char *i2c_eep_read_sequence(unsigned char DEV_ADR, unsigned int EEP_ADR, unsigned char n_r)
{	unsigned int temp;											//Hilfsvariable
	unsigned char byte_array[n_r];							//Array mit n_r Stellen
	i2c_start();
	i2c_write_byte(DEV_ADR);									//Sendet Gerteadresse R/W=0; Schreiben
	i2c_write_byte((unsigned char)(EEP_ADR>>8));				//Sendet MSB Teil der Speicheradresse
	i2c_write_byte((unsigned char)(EEP_ADR));					//Sendet LSB Teil der Spericheradresse
	i2c_start();												//Repeated Start, laut Datenblatt
	i2c_write_byte(DEV_ADR|0x01);								//Sendet Gerteadresse RW=1; LESEN
	for(temp=0; temp<=(n_r-1); temp++)						//Schleife zum Auslesen, (n_r-1) da letztes Byte nicht mehr ACK't wird
	{	byte_array[temp] = i2c_read_byte(1);					//Byte ins Array lesen (Read mit ACK, laut Datenblatt)
	}
	byte_array[n_r] = i2c_read_byte(0);						//Letztes Byte einlesen, kein ACK; 0
	i2c_stop();
	
}
