//Headerdatei fr IC EEPROM

#define TEILER 0								//0,1,2,3 
#define FAKTOR 40								//faktor = (FCPU/(SCL-16)/(2*4^TWPS)

//********************************************************************************************************************************************
//---------------------------------------------------- IC-Hilfsunktionen --------------------------------------------------------------------

//Funktion initialisiert die IC-Hardware mit SCL=100kHz
//bergabeparameter: keine
//Rckgabeparameter: 0 wenn alles OK
int i2c_ini(void);	

//Funktion generiert die Startbedingung im Mastermode
//bergabeparameter: keine
//Rckgabeparameter: 0 wenn alles OK
int i2c_start(void);

//Funktion generiert die Stoppbedingung im Mastermode
//bergabeparameter: keine
//Rckgabeparameter: 0 wenn alles OK
int i2c_stop(void);

//Funktion schreibt ein Byte auf den Bus und wartet, bis ACK kommt
//bergabeparameter: Datenbyte
//Rckgabeparameter: 0 wenn alles OK
int i2c_write_byte(unsigned char byte);

//Funktion liest ein Byte vom Bus 
//bergabeparameter: 0 wenn NACK, 1 wenn ACK
//Rckgabeparameter: gelesenes Byte
unsigned char i2c_read_byte(unsigned char ack_flag);


//*****************************************************************************************************************************************
//---------------------------------------------------- IC-Haupfunktionen -----------------------------------------------------------------

//Funktion schreibt ein Byte an eine Gerteadresse
//bergabeparameter: Gerteadresse (ohne RW), Byte
//Rckgabeparameter: 0 wenn alles ok
int i2c_write_byte_adr(unsigned char ADR, unsigned char byte);


//Funktion liest ein Byte von einer Gerteadresse
//bergabeparameter: Gerteadresse(ohne RW), 1 wenn beim lesen auf ein ACK gewartet werden soll, 0 wenn NACK
//Rckgabeparameter: gelsenes Byte
unsigned char i2c_read_byte_adr(unsigned char ADR, unsigned char ack_flag);


//********************************************************************************************************************************************
//--------------------------------------------------- IC-EEPROM-Funktionen ------------------------------------------------------------------

//Funktion liest die momentane Speicheradresse des EEPROMs aus. Speicherzeiger wird intern um eins erhht(Laut DB nur ein Byte, warum  1 Byte bei 64k??)
//bergabeparameter: Gerteadresse (ohne RW)
//Rckgabeparameter: Speicheradresse
unsigned char i2c_eep_readadr(unsigned char DEV_ADR);

//Funktion schreibt ein Byte an eine bestimmte Speicheradresse des EEPROMs
//bergabeparameter: Gerteadresse(ohne RW), EEPROM Speicheradresse, Pointer auf Byte
//Rckgabeparameter: 0 wenn alles OK
int i2c_eep_write_byte(unsigned char DEV_ADR, unsigned int EEP_ADR, unsigned char byte);

//Funktion schreibt eine Page an Bytes (max 128)
//bergabeparameter: Gerteadresse(ohne RW), EEPROM Speicheradresse, Pointer auf Byte, Anzahl der Bytes
//Rckgabeparameter: 0 wenn alles OK, 1 wenn mehr als 128 Bytes sind
int i2c_eep_write_page(unsigned char DEV_ADR, unsigned int EEP_ADR, unsigned char *byte, unsigned char n);

//Funktion liest einen Bereich des EEproms aus
//bergabeparameter: Gerteadresse(ohne RW), Speicheradresse des EEproms, Anzahl der zu lesenden BYtes
//Rckgabeparameter: Pointer auf Array
unsigned char *i2c_eep_read_sequence(unsigned char DEV_ADR, unsigned int EEP_ADR, unsigned char n_r);
